# ALMA Data Reduction Script

# Calibration
# ** Run uid___A002_Xdab261_X13a0a.ms.scriptForCalibration.py first
# to generate bandpass table
thesteps = []
step_title = {0: 'Import of the ASDM',
              1: 'Fix of SYSCAL table times',
              2: 'listobs',
              3: 'A priori flagging',
              4: 'Generation and time averaging of the WVR cal table',
              5: 'Generation of the Tsys cal table',
              6: 'Generation of the antenna position cal table',
              7: 'Application of the WVR, Tsys and antpos cal tables',
              8: 'Split out science SPWs and time average',
              9: 'Listobs, and save original flags',
              10: 'Initial flagging',
              11: 'Putting a model for the flux calibrator(s)',
              12: 'Save flags before bandpass cal',
              #13: 'Bandpass calibration', # use previous bandpass table instead
              14: 'Save flags before gain cal',
              15: 'Gain calibration',
              16: 'Save flags before applycal',
              17: 'Application of the bandpass and gain cal tables',
              18: 'Split out corrected column',
              19: 'Save flags after applycal'}

if 'applyonly' not in globals(): applyonly = False
try:
  print 'List of steps to be executed ...', mysteps
  thesteps = mysteps
except:
  print 'global variable mysteps not set.'
if (thesteps==[]):
  thesteps = range(0,len(step_title))
  print 'Executing all steps: ', thesteps

# The Python variable 'mysteps' will control which steps
# are executed when you start the script using
#   execfile('scriptForCalibration.py')
# e.g. setting
#   mysteps = [2,3,4]# before starting the script will make the script execute
# only steps 2, 3, and 4
# Setting mysteps = [] will make it execute all steps.

import re
import os
import casadef
import numpy as np

if cu.version()[0] != 5:
  sys.exit('ERROR: Please use CASA 5.x e.g. 5.1.1 as used to generate this script')

bandpassCalTable = 'uid___A002_Xdab261_X13a0a.ms.split.bandpass'

# CALIBRATE_AMPLI: 
# CALIBRATE_ATMOSPHERE: J0522-3627,OrionKL
# CALIBRATE_BANDPASS: 
# CALIBRATE_DIFFGAIN: 
# CALIBRATE_FLUX: 
# CALIBRATE_FOCUS: 
# CALIBRATE_PHASE: J0529-0519
# CALIBRATE_POINTING: J0522-3627,J0542-0913
# OBSERVE_CHECK: 
# OBSERVE_TARGET: OrionKL

# for plotting in plotcal
ants=['DA41','DA42','DA43','DA44','DA45','DA46','DA47','DA48','DA49','DA50','DA51','DA52','DA53','DA54','DA55','DA56','DA59','DA60','DA61','DA62','DA63','DA64','DA65','DV01','DV02','DV03','DV04','DV05','DV06','DV07','DV08','DV09','DV11','DV13','DV14','DV15','DV16','DV17','DV19','DV20','DV22','DV23','DV24','DV25']

ants4=np.array_split(np.array(ants),(ceil(len(ants)/4.)))

# Using reference antenna = DA55

# Import of the ASDM
mystep = 0
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  if os.path.exists('uid___A002_Xdab261_X1448a.ms') == False:
    importasdm('uid___A002_Xdab261_X1448a', asis='Antenna Station Receiver Source CalAtmosphere CalWVR CorrelatorMode SBSummary', bdfflags=True, lazy=False, process_caldevice=False)
    if not os.path.exists('uid___A002_Xdab261_X1448a.ms.flagversions'):
      print 'ERROR in importasdm. Output MS is probably not useful. Will stop here.'
      thesteps = []

# Fix of SYSCAL table times
mystep = 1
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  from recipes.almahelpers import fixsyscaltimes
  fixsyscaltimes(vis = 'uid___A002_Xdab261_X1448a.ms')

print "# A priori calibration"

# listobs
mystep = 2
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_Xdab261_X1448a.ms.listobs')
  listobs(vis = 'uid___A002_Xdab261_X1448a.ms',
    listfile = 'uid___A002_Xdab261_X1448a.ms.listobs')
  
  

# A priori flagging
mystep = 3
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  flagdata(vis = 'uid___A002_Xdab261_X1448a.ms',
    mode = 'manual',
    spw = '5~20',
    autocorr = True,
    flagbackup = False)
  
  flagdata(vis = 'uid___A002_Xdab261_X1448a.ms',
    mode = 'manual',
    intent = '*POINTING*,*ATMOSPHERE*',
    flagbackup = False)
  
  flagcmd(vis = 'uid___A002_Xdab261_X1448a.ms',
    inpmode = 'table',
    useapplied = True,
    action = 'plot',
    plotfile = 'uid___A002_Xdab261_X1448a.ms.flagcmd.png')
  
  flagcmd(vis = 'uid___A002_Xdab261_X1448a.ms',
    inpmode = 'table',
    useapplied = True,
    action = 'apply')
  

# Generation and time averaging of the WVR cal table
mystep = 4
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_Xdab261_X1448a.ms.wvr') 
  
  os.system('rm -rf uid___A002_Xdab261_X1448a.ms.wvrgcal') 
  
  mylogfile = casalog.logfile()
  casalog.setlogfile('uid___A002_Xdab261_X1448a.ms.wvrgcal')
  
  wvrgcal(vis = 'uid___A002_Xdab261_X1448a.ms',
    caltable = 'uid___A002_Xdab261_X1448a.ms.wvr',
    spw = [13, 15, 17, 19],
    toffset = 0,
    tie = ['OrionKL,J0529-0519'],
    statsource = 'OrionKL')
  
  casalog.setlogfile(mylogfile)
  
  os.system('mkdir uid___A002_Xdab261_X1448a.ms.wvr.plots')
  
  for a4 in ants4:
    ant=''
    for a in a4:
      ant=ant+','+a
    ant=ant[1:]; antstring=ant.replace(',','_')
    plotcal(caltable='uid___A002_Xdab261_X1448a.ms.wvr',
            xaxis='time',yaxis='phase',
            subplot=411,antenna=ant,iteration='antenna',
            showgui=False,
            figfile='uid___A002_Xdab261_X1448a.ms.wvr.plots/uid___A002_Xdab261_X1448a.ms.wvr.'+antstring+'.png')


# Generation of the Tsys cal table
mystep = 5
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_Xdab261_X1448a.ms.tsys') 
  gencal(vis = 'uid___A002_Xdab261_X1448a.ms',
    caltable = 'uid___A002_Xdab261_X1448a.ms.tsys',
    caltype = 'tsys')
  
  # Flagging edge channels
  
  flagdata(vis = 'uid___A002_Xdab261_X1448a.ms.tsys',
    mode = 'manual',
    spw = '13:0~0;63~63,15:0~0;63~63,17:0~0;63~63,19:0~0;63~63',
    flagbackup = False)
  
  os.system('mkdir uid___A002_Xdab261_X1448a.ms.tsys.plots')
  for ant in ants:
    for s in [0,1,2,3]:
      plotcal(caltable='uid___A002_Xdab261_X1448a.ms.tsys',
              xaxis='freq',yaxis='tsys',spw=str(13+2*s),
              subplot=221+s,antenna=ant,
              showgui=False,
              figfile='uid___A002_Xdab261_X1448a.ms.tsys.plots/uid___A002_Xdab261_X1448a.ms.tsys.'+ant+'.png')

  #plotms(vis='uid___A002_Xdab261_X1448a.ms.tsys',
  #       xaxis='freq', yaxis='tsys', field='OrionKL', # target
  #       plotfile='uid___A002_Xdab261_X1448a.ms.tsys.plots/uid___A002_Xdab261_X1448a.ms.tsys_atm.png',
  #       showgui=False,showatm=True)

  

# Generation of the antenna position cal table
mystep = 6
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  # Warning: no baseline run found for following antenna(s): ['DA48', 'DA62', 'DV22', 'DA52', 'DA54', 'DV07', 'DV24'].
  
  os.system('rm -rf uid___A002_Xdab261_X1448a.ms.antpos') 
  gencal(vis = 'uid___A002_Xdab261_X1448a.ms',
    caltable = 'uid___A002_Xdab261_X1448a.ms.antpos',
    caltype = 'antpos',
    antenna = 'DA41',
  #  parameter = [])
    parameter = [0,0,0])
  
  
  # antenna x_offset y_offset z_offset total_offset baseline_date
  

# Application of the WVR, Tsys and antpos cal tables
mystep = 7
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  clearstat()  
  
  from recipes.almahelpers import tsysspwmap
  tsysmap = tsysspwmap(vis = 'uid___A002_Xdab261_X1448a.ms', tsystable = 'uid___A002_Xdab261_X1448a.ms.tsys', tsysChanTol = 1)
  
  
  
  # Note: J0542-0913 didn't have any Tsys measurement, and I couldn't find any close measurement. But this is not a science target, so this is probably Ok.
  
  # Note: J0529-0519 didn't have any Tsys measurement, so I used the one made on OrionKL. This is probably Ok.
  
  applycal(vis = 'uid___A002_Xdab261_X1448a.ms',
    field = '1',
    spw = '13,15,17,19',
    gaintable = ['uid___A002_Xdab261_X1448a.ms.tsys', 'uid___A002_Xdab261_X1448a.ms.wvr', 'uid___A002_Xdab261_X1448a.ms.antpos'],
    gainfield = ['2', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = True,
    flagbackup = False)
  
  
  
  applycal(vis = 'uid___A002_Xdab261_X1448a.ms',
    field = '2~39,41~47',
    spw = '13,15,17,19',
    gaintable = ['uid___A002_Xdab261_X1448a.ms.tsys', 'uid___A002_Xdab261_X1448a.ms.wvr', 'uid___A002_Xdab261_X1448a.ms.antpos'],
    gainfield = ['2', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = True,
    flagbackup = False)
  
  
  
  applycal(vis = 'uid___A002_Xdab261_X1448a.ms',
    field = '40',
    spw = '13,15,17,19',
    gaintable = ['uid___A002_Xdab261_X1448a.ms.tsys', 'uid___A002_Xdab261_X1448a.ms.wvr', 'uid___A002_Xdab261_X1448a.ms.antpos'],
    gainfield = ['40', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = True,
    flagbackup = False)
  

  # plot weights  
  #for s in ['13','15','17','19']:
  #          plotms(vis = 'uid___A002_Xdab261_X1448a.ms', 
  #                 xaxis = 'time', yaxis = 'weight', spw = s,
  #                 coloraxis = 'field', plotfile =  'uid___A002_Xdab261_X1448a.ms' + '_weights_spw'+s+'.png', 
  #                 showgui=False,overwrite = True)
    

# Split out science SPWs and time average
mystep = 8
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_Xdab261_X1448a.ms.split') 
  os.system('rm -rf uid___A002_Xdab261_X1448a.ms.split.flagversions') 
  
  split(vis = 'uid___A002_Xdab261_X1448a.ms',
    outputvis = 'uid___A002_Xdab261_X1448a.ms.split',
    datacolumn = 'corrected',
    spw = '13,15,17,19',
    keepflags = True)
  
  

print "# Calibration"

# Listobs, and save original flags
mystep = 9
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_Xdab261_X1448a.ms.split.listobs')
  listobs(vis = 'uid___A002_Xdab261_X1448a.ms.split',
    listfile = 'uid___A002_Xdab261_X1448a.ms.split.listobs')
  
  
  if not os.path.exists('uid___A002_Xdab261_X1448a.ms.split.flagversions/Original.flags'):
    flagmanager(vis = 'uid___A002_Xdab261_X1448a.ms.split',
      mode = 'save',
      versionname = 'Original')
    

# Initial flagging
mystep = 10
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  # Flagging shadowed data
  
  flagdata(vis = 'uid___A002_Xdab261_X1448a.ms.split',
    mode = 'shadow',
    flagbackup = False)
  
  # Flagging edge channels
  
  flagdata(vis = 'uid___A002_Xdab261_X1448a.ms.split',
    mode = 'manual',
    spw = '0:0~3;60~63,1:0~3;60~63,2:0~3;60~63,3:0~3;60~63',
    flagbackup = False)
  
  

# Putting a model for the flux calibrator(s)
mystep = 11
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  setjy(vis = 'uid___A002_Xdab261_X1448a.ms.split',
    standard = 'manual',
    field = 'J0522-3627',
    scalebychan = False,
    usescratch  = True,
    fluxdensity = [3.8, 0, 0, 0],
    spix = -0.266,
    reffreq = '233.0GHz')

# Save flags before bandpass cal
mystep = 12
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_Xdab261_X1448a.ms.split',
    mode = 'save',
    versionname = 'BeforeBandpassCalibration')
  
  

# Bandpass calibration
#mystep = 13
#if(mystep in thesteps):
#  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
#  print 'Step ', mystep, step_title[mystep]

#Not used - bandpass table from first execution used

  # Note: there are no bandpass calibrator, I'm picking a phase calibrator.
#  os.system('rm -rf uid___A002_Xdab261_X1448a.ms.split.ap_pre_bandpass') 
#  
#  gaincal(vis = 'uid___A002_Xdab261_X1448a.ms.split',
#    caltable = 'uid___A002_Xdab261_X1448a.ms.split.ap_pre_bandpass',
#    field = 'J0522-3627', # J0522-3627
#    spw = '0:0~63,1:0~63,2:0~63,3:0~63',
#    scan = '15,30,38',
#    solint = 'int',
#    refant = 'DA55',
#    calmode = 'p')
#  
#  if applyonly != True: es.checkCalTable('uid___A002_Xdab261_X1448a.ms.split.ap_pre_bandpass', msName='uid___A002_Xdab261_X1448a.ms.split', interactive=False) 
#  
#  os.system('rm -rf uid___A002_Xdab261_X1448a.ms.split.bandpass') 
#  bandpass(vis = 'uid___A002_Xdab261_X1448a.ms.split',
#    caltable = 'uid___A002_Xdab261_X1448a.ms.split.bandpass',
#    field = 'J0522-3627', # J0522-3627
#    scan = '15,30,38',
#    solint = 'inf',
#    combine = 'scan',
#    refant = 'DA55',
#    solnorm = True,
#    bandtype = 'B',
#    gaintable = 'uid___A002_Xdab261_X1448a.ms.split.ap_pre_bandpass')
  
#  if applyonly != True: es.checkCalTable('uid___A002_Xdab261_X1448a.ms.split.bandpass', msName='uid___A002_Xdab261_X1448a.ms.split', interactive=False) 
  

# Save flags before gain cal
mystep = 14
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_Xdab261_X1448a.ms.split',
    mode = 'save',
    versionname = 'BeforeGainCalibration')
  
  

# Gain calibration
mystep = 15
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_Xdab261_X1448a.ms.split.phase_int') 
  gaincal(vis = 'uid___A002_Xdab261_X1448a.ms.split',
    caltable = 'uid___A002_Xdab261_X1448a.ms.split.phase_int',
    field = '1~1,40~40', # J0529-0519,J0522-3627
    solint = 'int',
    refant = 'DA55',
    gaintype = 'G',
    calmode = 'p',
    gaintable = bandpassCalTable)
    
  os.system('mkdir uid___A002_Xdab261_X1448a.ms.split.phase_int.plots')
  for ant in ants:
    plotcal(caltable='uid___A002_Xdab261_X1448a.ms.split.phase_int',
            figfile='uid___A002_Xdab261_X1448a.ms.split.phase_int.plots/uid___A002_Xdab261_X1448a.ms.split.phase_int.'+ant+'.png',
            xaxis='time',yaxis='phase',
            showgui=False,
            subplot=411,antenna=ant,iteration='spw')

  os.system('rm -rf uid___A002_Xdab261_X1448a.ms.split.ampli_inf')
  gaincal(vis = 'uid___A002_Xdab261_X1448a.ms.split',
    caltable = 'uid___A002_Xdab261_X1448a.ms.split.ampli_inf',
    field = '1~1,40~40', # J0522-3627,J0529-0519
    solint = 'inf',
    refant = 'DA55',
    gaintype = 'T',
    calmode = 'a',
    gaintable = [bandpassCalTable, 'uid___A002_Xdab261_X1448a.ms.split.phase_int'])
  
  os.system('mkdir uid___A002_Xdab261_X1448a.ms.split.ampli_inf.plots')
  for ant in ants:
    plotcal(caltable='uid___A002_Xdab261_X1448a.ms.split.ampli_inf',
            xaxis='time',yaxis='amp',
            subplot=411,antenna=ant,iteration='spw',
            showgui=False,
            figfile='uid___A002_Xdab261_X1448a.ms.split.ampli_inf.plots/uid___A002_Xdab261_X1448a.ms.split.ampli_inf.'+ant+'.png') 

  os.system('rm -rf uid___A002_Xdab261_X1448a.ms.split.flux_inf')
  os.system('rm -rf uid___A002_Xdab261_X1448a.ms.split.fluxscale')
  mylogfile = casalog.logfile()
  casalog.setlogfile('uid___A002_Xdab261_X1448a.ms.split.fluxscale')

  fluxscaleDict = fluxscale(vis = 'uid___A002_Xdab261_X1448a.ms.split',
                            caltable = 'uid___A002_Xdab261_X1448a.ms.split.ampli_inf',
                            fluxtable = 'uid___A002_Xdab261_X1448a.ms.split.flux_inf',
                            reference = 'J0522-3627') # J0522-3627

  casalog.setlogfile(mylogfile)
  
  
  os.system('rm -rf uid___A002_Xdab261_X1448a.ms.split.phase_inf') 
  gaincal(vis = 'uid___A002_Xdab261_X1448a.ms.split',
    caltable = 'uid___A002_Xdab261_X1448a.ms.split.phase_inf',
    field = '0~2', # J0522-3627,J0423-0120,J0529-0519
    solint = 'inf',
    refant = 'DA55',
    gaintype = 'G',
    calmode = 'p',
    gaintable = bandpassCalTable)
  
  os.system('mkdir uid___A002_Xdab261_X1448a.ms.split.phase_inf.plots')
  for ant in ants:
    plotcal(caltable='uid___A002_Xdab261_X1448a.ms.split.phase_inf',
            xaxis='time',yaxis='phase',
            subplot=411,antenna=ant,iteration='spw',
            showgui=False,
            figfile='uid___A002_Xdab261_X1448a.ms.split.phase_inf.plots/uid___A002_Xdab261_X1448a.ms.split.phase_inf.'+ant+'.png')
  

# Save flags before applycal
mystep = 16
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_Xdab261_X1448a.ms.split',
    mode = 'save',
    versionname = 'BeforeApplycal')
  
  

# Application of the bandpass and gain cal tables
mystep = 17
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  for i in ['40']: # J0522-3627
    applycal(vis = 'uid___A002_Xdab261_X1448a.ms.split',
      field = str(i),
      gaintable = [bandpassCalTable, 'uid___A002_Xdab261_X1448a.ms.split.phase_int', 'uid___A002_Xdab261_X1448a.ms.split.flux_inf'],
      gainfield = ['J0522-3627', 'J0529-0519', 'J0529-0519' ],
      interp = 'linear,linear',
      calwt = True,
      flagbackup = False)
  
  applycal(vis = 'uid___A002_Xdab261_X1448a.ms.split',
    field = '1,2~39,41~47', # OrionKL
    gaintable = [bandpassCalTable, 'uid___A002_Xdab261_X1448a.ms.split.phase_inf', 'uid___A002_Xdab261_X1448a.ms.split.flux_inf'],
    gainfield = ['J0522-3627', '1', '1'], # J0529-0519
    interp = 'linear,linear',
    calwt = True,
    flagbackup = False)
  

# Split out corrected column
mystep = 18
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_Xdab261_X1448a.ms.split.cal') 
  os.system('rm -rf uid___A002_Xdab261_X1448a.ms.split.cal.flagversions') 
  
  listOfIntents = ['CALIBRATE_PHASE#ON_SOURCE',
   'CALIBRATE_POLARIZATION#ON_SOURCE',
   'CALIBRATE_WVR#AMBIENT',
   'CALIBRATE_WVR#HOT',
   'CALIBRATE_WVR#OFF_SOURCE',
   'CALIBRATE_WVR#ON_SOURCE',
   'OBSERVE_TARGET#ON_SOURCE']
  
  split(vis = 'uid___A002_Xdab261_X1448a.ms.split',
    outputvis = 'uid___A002_Xdab261_X1448a.ms.split.cal',
    datacolumn = 'corrected',
    intent = ','.join(listOfIntents),
    keepflags = True)
  
  

# Save flags after applycal
mystep = 19
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_Xdab261_X1448a.ms.split.cal',
    mode = 'save',
    versionname = 'AfterApplycal')
  
  

